//
//	timeCorrelator.cc: test the TimeCorrelator class
//

#include <iostream>
#include <cmath>
#include <cassert>
#include "TimeCorrelator.h"
#include "OalServer.h"

int main()
{
	//
	//	instantiate TimeCorrelator with dummy ODF TCF - this includes
	//	an OBT wrap-around and the introduction of leap second in UTC
	//	on 1987-12-31T23:59:60
	//
	const Odf *odf = oalServer.newOdf("../odf");
	const OdfSet *tcf = odf->selectSet(Xmm::SC, Xmm::TIME_CORRELATION);

	TimeCorrelator::DataInterpolatorSpecs
			obtutcspec = { 1, 1.e-8, .9, true },
											// max degree 1, RMS: 1.e-6,
											// can remove 90% of points
			ertutcspec = { 1, 1.e-2, .9, true };
											// max degree 1, RMS: 1.e-2,
											// can remove 90% of points
	TimeCorrelator::TimeCorrelatorSpecs tSpec;
	tSpec.range = TimeCorrelator::ALL;
	tSpec.source = TimeCorrelator::TCS;
	tSpec.method = TimeCorrelator::TCS_ONLY;
	
	TimeCorrelator tcorr(tcf, tSpec, obtutcspec, ertutcspec, 
			     STime("1987-12-31T23:59:50"));	// make this the base time

	cout.precision(16);
	double obt1 = 2147483645.0;
	double d1 = tcorr.toMissionEllapsedTime(obt1);
											// that should be close to 6.456s
	assert(fabs(d1/6.456-1.) <= 1.e-8);

	for (double delta=1.; delta<=1000; delta+=5) {
		double d = tcorr.toMissionEllapsedTime(obt1 + delta);
		//cout<<d<<" "<<fabs((d-d1)/delta - 1.)<<endl;
		assert(fabs((d-d1)/delta - 1.) < 0.011);
	}

	assert(fabs(tcorr.toMissionEllapsedTime("1987-12-31T23:59:59")/9.8-1.)
			<.02);
	assert(fabs(tcorr.toMissionEllapsedTime("1987-12-31T23:59:60.1")/10.9-1.)
			<.02);
	assert(fabs(tcorr.toMissionEllapsedTime("1988-01-01")/11.8-1.)<.02);

	//
	//	instantiate a second time correlator with a perfectly linear TCF
	//	precision should be much higher
	//
	TimeCorrelator tcorr2("timecorr.fits", tSpec, obtutcspec, ertutcspec, STime("1998-01-01"));

	obt1 = 1.0;
	const double dexpt = 3456000.;
	d1 = tcorr2.toMissionEllapsedTime(obt1);// that should be equal to 68 days
											// = 3456000s
	assert(fabs(d1/dexpt-1.) <= 1.e-10);
	for (double delta=1.; delta<=1000.; delta+=5.345) {
		double d = tcorr2.toMissionEllapsedTime(obt1 + delta);
		//cout<<d<<" "<<fabs((d-d1)/delta - 1.)<<endl;
		assert(fabs((d-d1)/delta - 1.) < 1.e-10);
	}

	assert(fabs(tcorr2.toMissionEllapsedTime("1998-02-01T00:00:00")/dexpt-1.)
			<1.e-10);
	STime t(STime("1998-03-01T23:12:23.456"));
	double diff = t - STime("1998-02-01");
	assert(fabs((tcorr2.toMissionEllapsedTime(t)-dexpt)/diff-1.)<1.e-10);

    diff = STime("2000-01-01") - STime("1998-01-01");
    cout<<" Diff between 2000 and 1998 is "<<diff;

	cout<<"All tests successfully passed.\n";
	return 0;
}

